#!/bin/bash
#
# Install Torque on cluster.
#
# Eliot Eshelman <eliot@microway.com>
# Last updated: 10/27/2005
#
#


function failure() {
  if [ $1 -gt 0 ]; then
    # failure
    echo "An error occured during $2.  Please see the file '$2.microway.log'";
    echo "in the Torque source directory.";
    exit 1;
  fi
}


PBS_VER=torque-1.2.0p4;
MAUI_VER=maui-3.2.6p13;
ARCH=`uname -m`;
MASTER=`hostname -s`;
if [ "$NODES" != "" ]
  then
  Nodes=$NODES
  else
  Nodes=/etc/nodes
fi


echo "Installing $PBS_VER";
echo "This installation script relies on rsh/rcom/rcpf!";
echo;


# Configure
WD=`pwd`;

cp -a $PBS_VER /tmp
cd /tmp/$PBS_VER;


[ ! -f suse10-patched ] && [ -f /etc/SuSE-release ] && grep -q 10.0 /etc/SuSE-release && patch -p1 < $WD/patch-for-suse10.diff && touch suse10-patched

if [ $ARCH = 'x86_64' ]; then
  # 64 bit install

  echo "Configuring PBS for 64 bit system...";
  ./configure --enable-docs  --enable-gui --set-server-home=/var/spool/pbs --set-cflags="-m64 -L/lib64 -L/usr/lib64 -L/usr/X11R6/lib64 -include errno.h" &> $WD/configure.microway.log
else
  # 32 bit install
  echo "Configuring PBS for 32 bit system...";
  ./configure --enable-docs  --enable-gui  --set-server-home=/var/spool/pbs &> $WD/configure.microway.log
fi

# check for configuration failure
failure $? configure;


# Compile
echo "Compiling PBS...";
make &> $WD/make.microway.log;

# check for compilation failure
failure $? make;


# Install
echo "Installing PBS on localhost...";

make install &> $WD/install.microway.log;
failure $? install;


echo "Installing PBS across cluster...";
cd ../;
tar cf pbs.install.tar torque-1.2.0p4;
rcpf-nodes pbs.install.tar /tmp;
> /tmp/installing_pbs.tmp &> /dev/null;
rcom-nodes-parallel "cd /tmp; tar xf pbs.install.tar 2>&1 | grep -vi 'stamp'; cd $PBS_VER; make install &> /dev/null; cd ../; rm -Rf $PBS_VER pbs.install.tar; MY_HOSTNAME=\`hostname\`; rsh $MASTER \"echo \$MY_HOSTNAME >> /tmp/installing_pbs.tmp\";"

# Wait until nodes are finished installing...
echo "Waiting for nodes to finish installation.";
START_TIME=`date +%s`;
TIMEOUT=$((START_TIME+120));
while [ `wc /tmp/installing_pbs.tmp | awk '{print $1}'` -lt `cat $Nodes | grep -vw $MASTER | wc | awk '{print $1}'` ]; do
  echo -n ".";
  sleep 3s;

  if [ `date +%s` -gt $TIMEOUT ]; then
    echo "Timeout.  Not all nodes finished the installation!";
    echo "See 'finished_installation.txt' for the list of nodes that did finish.";
    cp /tmp/installing_pbs.tmp $WD/finished_installation.txt;
    break;
  fi

done
echo;

cd $WD

rcpf pbs.rc /etc/init.d/pbs;
rm -Rf /tmp/installing_pbs.tmp pbs.install.tar;


# Configure
echo "Configuring PBS...";
cp pbs.conf.master /etc/pbs.conf;
rcpf-nodes pbs.conf.node /etc/pbs.conf;

rcom-parallel "echo $MASTER > /var/spool/pbs/server_name";
rcom-parallel "echo \$\clienthost $MASTER > /var/spool/pbs/mom_priv/config";
cat /etc/nodes | grep -vw $MASTER | grep -v localhost | sed 's/$/\ np=2/' > /var/spool/pbs/server_priv/nodes;


# Add pbs to boot order
if [ -x /sbin/chkconfig ]; then
  rcom-parallel "/sbin/chkconfig --add pbs" &> /dev/null;
elif [ -x /sbin/rc-update ]; then
  rcom-parallel "/sbin/rc-update add pbs default" &> /dev/null;
else
  echo;
  echo "Unable to add PBS to the boot order.  You will need to manually add PBS";
  echo "to the boot order of your master and nodes.";
fi

# Start pbs
# Need to create the server database and then shutdown server.
/usr/local/sbin/pbs_server -t create;
if [ -x /sbin/service ]; then
  /sbin/service pbs stop &> /dev/null;
  killall pbs_server &> /dev/null;
  /sbin/service pbs start &> $WD/service.pbs.start.master;
  rcom-nodes-parallel "/sbin/service pbs start" &> $WD/service.pbs.start.nodes;
elif [ -x /etc/init.d/pbs ]; then
  /etc/init.d/pbs stop &> /dev/null;
  killall pbs_server &> /dev/null;
  /etc/init.d/pbs start &> $WD/service.pbs.start.master;
  rcom-nodes-parallel "/etc/init.d/pbs start" &> $WD/service.pbs.start.nodes;
else
  killall pbs_server &> /dev/null;
  echo;
  echo "Unable to start PBS daemons.  You will need to manually start PBS.";
fi


echo "Torque installation complete.";
echo;


# Configure Maui
echo "Installing $MAUI_VER...";
cd $MAUI_VER;
if [ $ARCH = 'x86_64' ]; then
  # 64 bit install
  echo "Configuring Maui for 64 bit system...";
  CFLAGS="-m64";
  LDFLAGS="-L/lib64 -L/usr/lib64";
else
  # 32 bit install
  echo "Configuring Maui for 32 bit system...";
  CFLAGS="-m32";
  LDFLAGS="-L/lib -L/usr/lib";
fi

echo "Killing any leftover maui processes before installing..."
killall -9 -w maui

./configure --prefix=/usr/local/maui/ --with-pbs &> $WD/configure_maui.microway.log;

# check for configuration failure
failure $? configure_maui;

# Compile Maui
echo "Compiling Maui...";
make &> $WD/make_maui.microway.log;

# check for compilation failure
failure $? make_maui;

# Install Maui
echo "Installing Maui...";
make install &> $WD/install_maui.microway.log;

# check for installation failure
failure $? install_maui;


# Configure Maui.
echo "You will need to configure Maui for your particular needs.";
echo "Please see /usr/local/maui/maui.cfg for configuration options.";

echo;
echo "Maui installation complete.";


echo;
echo "Installation complete!  Please check for errors above.";
echo "You can run 'pbsnodes -a' to test the PBS install.";

echo "Once you verify with pbsnodes -a, please cd to queue-setup and follow the procedure to initialize your queues."

rm -rf /tmp/$PBS_VER
